package octoshape.android.mediaplayer;

import android.app.Activity;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.google.android.gms.cast.MediaInfo;
import com.google.android.gms.cast.MediaMetadata;
import com.google.android.gms.cast.MediaStatus;
import octoshape.android.CastApplication;
import octoshape.android.R;
import com.google.android.libraries.cast.companionlibrary.cast.VideoCastManager;
import com.google.android.libraries.cast.companionlibrary.cast.callbacks.VideoCastConsumerImpl;
import com.google.android.libraries.cast.companionlibrary.cast.exceptions.CastException;
import com.google.android.libraries.cast.companionlibrary.cast.exceptions.NoConnectionException;
import com.google.android.libraries.cast.companionlibrary.cast.exceptions.TransientNetworkDisconnectionException;
import com.google.android.libraries.cast.companionlibrary.widgets.MiniController;


import java.util.Timer;
import java.util.TimerTask;

import octoshape.android.stream.StreamPlayerWrapper;

/**
 * Wrapper class for playing remotely on Google Cast devices.
 * Created by dota on 19-02-2015.
 */
public class RemotePlayerActivity implements IMediaPlayer {

    public static final String TAG = "RemotePlayerWrapper";
    VideoCastManager mCastManager;

    private MediaInfo mediaInfo;
    private String stream;
    private Timer mRemotePlayerStatusTimer;
    private Handler mHandler;
    private PlaybackState state;

    public RemotePlayerActivity(String stream, MediaInfo mediaInfo) {
        Log.d(TAG, "New RemotePlayerActivity created for " + stream);

        this.mediaInfo = mediaInfo;
        this.stream = stream;
        this.state = null;
        mCastManager = CastApplication.getCastManager();
        setCookie(mediaInfo);
    }

    private VideoCastConsumerImpl castConsumer = new VideoCastConsumerImpl() {
        public void onRemoteMediaPlayerStatusUpdated(){
            PlaybackState _state = PlaybackState.IDLE_UNKNOWN;

            if (mCastManager.isConnected()) {
                _state = getPlaybackState();
            }

            if (_state != state || state == null) {
                Log.d(TAG, "VideoCastConsumer ... new state: " + _state);
                if (_state ==  PlaybackState.PLAYING) {
                    CastApplication.getStreamManager().playbackStarted(RemotePlayerActivity.this.stream);
                } else if (_state == PlaybackState.IDLE_FINISHED) {
                    CastApplication.getStreamManager().playbackFinished(RemotePlayerActivity.this.stream);
                }
            }
            state = _state;
        }
    };

    public void startPlaying(Activity activity, int position, boolean play) {
//        MiniController mini = (MiniController) activity.findViewById(R.id.miniController1);
//        mini.setTitle(mediaInfo.getMetadata().getString(MediaMetadata.KEY_TITLE));
        mCastManager.addVideoCastConsumer(castConsumer);
        mCastManager.startVideoCastControllerActivity(activity,
                mediaInfo, position, play);
        restartTimer();
    }

    public void startPlaying(Context context, int position, boolean play) {
        mCastManager.addVideoCastConsumer(castConsumer);
        mCastManager.startVideoCastControllerActivity(context,
                mediaInfo, position, play);
        restartTimer();
    }

    @Override
    public PlaybackState getPlaybackState() {
        PlaybackState result = PlaybackState.PAUSED;

        if (mCastManager.isConnected()) {
            switch (mCastManager.getPlaybackStatus()) {
                case MediaStatus.PLAYER_STATE_BUFFERING:
                    result = PlaybackState.BUFFERING;
                    break;
                case MediaStatus.PLAYER_STATE_PAUSED:
                    result = PlaybackState.PAUSED;
                    break;
                case MediaStatus.PLAYER_STATE_PLAYING:
                    result = PlaybackState.PLAYING;
                    break;
                case MediaStatus.PLAYER_STATE_IDLE:
                    switch (mCastManager.getIdleReason()) {
                        case MediaStatus.IDLE_REASON_INTERRUPTED:
                            result = PlaybackState.IDLE_UNKNOWN;
                            break;
                        case MediaStatus.IDLE_REASON_CANCELED:
                            result = PlaybackState.IDLE_UNKNOWN;
                            break;
                        case MediaStatus.IDLE_REASON_FINISHED:
                            result = PlaybackState.IDLE_FINISHED;
                            break;
                        case MediaStatus.IDLE_REASON_ERROR:
                            result = PlaybackState.IDLE_UNKNOWN;
                            break;
                    }
                    break;
                default:
                    result = PlaybackState.IDLE_UNKNOWN;
            }
        }

        return result;
    }

    @Override
    public int getPlaybackPosition() {
        int result = 0;
        if (mCastManager.isConnected()) {
            try {
                result = (int) mCastManager.getCurrentMediaPosition();
            } catch (TransientNetworkDisconnectionException e) {
                Log.d(TAG, "Exception: " + e.getMessage());
                e.printStackTrace();
            } catch (NoConnectionException e) {
                Log.d(TAG, "Exception: " + e.getMessage());
                e.printStackTrace();
            }
        } else {
            Log.d(TAG, "Cast manager: NOT CONNECTED");
        }

        Log.d(TAG, "playback position: " + result);

        return result;
    }

    @Override
    public void stopPlay() {
        Log.d(TAG, "Stopping Remote Player Activity for " + stream);

        try {
            stopTimer();
            if (mCastManager.isConnected() && mCastManager.isRemoteMediaLoaded()) {
                mCastManager.stop();
            }

            CastApplication.getStreamManager().playbackFinished(stream);
            mCastManager.removeVideoCastConsumer(castConsumer);
        } catch (CastException e) {
            e.printStackTrace();
        } catch (TransientNetworkDisconnectionException e) {
            e.printStackTrace();
        } catch (NoConnectionException e) {
            e.printStackTrace();
        }
    }

    private void stopTimer() {
        Log.d(TAG, "Stopped Timer");
        if (null != mRemotePlayerStatusTimer) {
            mRemotePlayerStatusTimer.cancel();
        }
    }

    private void restartTimer() {
        stopTimer();
        mRemotePlayerStatusTimer = new Timer();
        mRemotePlayerStatusTimer.scheduleAtFixedRate(new UpdateRemotePlayerStatus(), 100, 1000);
        Log.d(TAG, "Restarted Timer");
    }

    @Override
    public void updatedMediaInfo(MediaInfo mediaInfo) {

        this.mediaInfo = mediaInfo;
        setCookie(mediaInfo);

        loadRemoteMediaInfo();
    }

    @Override
    public void updateDuration(long duration) {
        mediaInfo = new MediaInfo.Builder(mediaInfo.getContentId())
                .setMediaTracks(mediaInfo.getMediaTracks())
                .setContentType(mediaInfo.getContentType())
                .setCustomData(mediaInfo.getCustomData())
                .setMetadata(mediaInfo.getMetadata())
                .setStreamType(mediaInfo.getStreamType())
                .setTextTrackStyle(mediaInfo.getTextTrackStyle())
                .setStreamDuration(duration).build();

        loadRemoteMediaInfo();
    }

    private void loadRemoteMediaInfo() {
        try {
            Log.d(TAG, "Updated mediaInfo " + mediaInfo.toJson().toString());
            mCastManager.loadMedia(mediaInfo, true, 0);

            restartTimer();
        } catch (TransientNetworkDisconnectionException e) {
            e.printStackTrace();
        } catch (NoConnectionException e) {
            e.printStackTrace();
        }
    }


    /**
     * Sends the Google Cast custom receiver a custom message with the key. The custom receiver
     * will then set the cookie on the Cast device to allow it to play the streams.
     * @param mediaInfo
     */
    private void setCookie(MediaInfo mediaInfo) {
        if (mediaInfo.getCustomData() != null && mediaInfo.getCustomData().has(StreamPlayerWrapper.CC_STRING) && mCastManager.isConnected()){
            Log.d(TAG, "Cookie: " + mediaInfo.getCustomData().optString(StreamPlayerWrapper.CC_STRING));
            try{
                mCastManager.sendDataMessage(mediaInfo.getCustomData().optString(StreamPlayerWrapper.CC_STRING));
            } catch (TransientNetworkDisconnectionException e) {
                e.printStackTrace();
            } catch (NoConnectionException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Checks the status of the remote media player, and informs the stream manager if it's in playback or finished.
     * This is needed for reporting to the server. In case the playback is finished, we want to close the stream.
     */
    private class UpdateRemotePlayerStatus extends TimerTask {
        @Override
        public void run() {
//            Looper.prepare();
//            mHandler = new Handler();
//            mHandler.post(new Runnable() {
//                @Override
//                public void run() {
//                    PlaybackState _state = PlaybackState.IDLE_UNKNOWN;
//                    try {
//                        MediaInfo remoteInfo = mCastManager.getRemoteMediaInformation();
//                        Log.d(TAG, "Remote media Info: " + remoteInfo);
//                    } catch (TransientNetworkDisconnectionException e) {
//                        e.printStackTrace();
//                    } catch (NoConnectionException e) {
//                        e.printStackTrace();
//                    }
//
//                    if (mCastManager.isConnected()) {
//                        _state = getPlaybackState();
//                        Log.d(TAG, "Stat: " +  _state);
//                    }
//
//                    if (_state == PlaybackState.PLAYING) {
//                            Log.d(TAG, "Looper ... playing");
//                            CastApplication.getStreamManager().playbackStarted(stream); //, !notifyStatSystemPlaybackStarted);
//                        state = _state;
//                    } else if (_state == PlaybackState.IDLE_FINISHED && state != null){
//                        state = _state;
//                        stopTimer();
//                        Log.d(TAG, "FINISHED " + stream);
//                        CastApplication.getStreamManager().playbackFinished(stream); //, !notifyStatSystemPlaybackStarted);
//                    }
//
//
//                }
//            });
//            Looper.loop();
        }
    }
}
